import { __decorate, __metadata, __param } from 'tslib';
import { Injectable, Inject, NgModule } from '@angular/core';
import { HttpHeaders, HttpClient, HttpClientModule } from '@angular/common/http';
import { BehaviorSubject } from 'rxjs';
import { lib, PBKDF2, AES, pad, mode, enc } from 'crypto-js';
import 'rxjs/add/operator/map';

let CryptService = class CryptService {
    constructor() {
        this.keySize = 256;
        this.ivSize = 128;
        this.saltSize = 256;
        this.iterations = 1000;
    }
    encrypt(msg, pass) {
        const salt = lib.WordArray.random(this.saltSize / 8);
        const key = PBKDF2(pass, salt, {
            keySize: this.keySize / 32,
            iterations: this.iterations
        });
        const iv = lib.WordArray.random(this.ivSize / 8);
        const encrypted = AES.encrypt(msg, key, {
            iv: iv,
            padding: pad.Pkcs7,
            mode: mode.CBC
        });
        const encryptedHex = this.base64ToHex(encrypted.toString());
        const base64result = this.hexToBase64(salt + iv + encryptedHex);
        return base64result;
    }
    decrypt(transitmessage, pass) {
        if (transitmessage.toString() === '{}') {
            return JSON.stringify({});
        }
        const hexResult = this.base64ToHex(transitmessage);
        const salt = enc.Hex.parse(hexResult.substr(0, 64));
        const iv = enc.Hex.parse(hexResult.substr(64, 32));
        const encrypted = this.hexToBase64(hexResult.substring(96));
        const key = PBKDF2(pass, salt, {
            keySize: this.keySize / 32,
            iterations: this.iterations
        });
        const decrypted = AES.decrypt(encrypted, key, {
            iv: iv,
            padding: pad.Pkcs7,
            mode: mode.CBC
        });
        return decrypted.toString(enc.Utf8);
    }
    hexToBase64(str) {
        return btoa(String.fromCharCode.apply(null, str.replace(/\r|\n/g, "").replace(/([\da-fA-F]{2}) ?/g, "0x$1 ").replace(/ +$/, "").split(" ")));
    }
    base64ToHex(str) {
        for (var i = 0, bin = atob(str.replace(/[ \r\n]+$/, "")), hex = []; i < bin.length; ++i) {
            let tmp = bin.charCodeAt(i).toString(16);
            if (tmp.length === 1)
                tmp = "0" + tmp;
            hex[hex.length] = tmp;
        }
        return hex.join("");
    }
};
CryptService = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [])
], CryptService);

let CryptHttpService = class CryptHttpService {
    constructor(http, crypt) {
        this.http = http;
        this.crypt = crypt;
        this.secret = '';
        this.isOnLoad = new BehaviorSubject(this.running);
    }
    encrypt(content) {
        const ciphertext = this.crypt.encrypt(JSON.stringify(content), this.secret);
        return ciphertext;
    }
    decrypt(content) {
        if (!this.cryptInactive) {
            if (content.text) {
                content = content.text();
            }
            const bytes = this.crypt.decrypt(content.toString(), this.secret);
            try {
                const decrypted = JSON.parse(bytes);
                this.isOnLoad.next(false);
                return decrypted;
            }
            catch (err) {
                this.isOnLoad.next(false);
                return { error: 'Can not decypted the content' };
            }
        }
        this.isOnLoad.next(false);
        if (!this.warnInactive) {
            this.warn();
        }
        return content;
    }
    setCryptHeaders(headers) {
        if (!headers) {
            headers = new HttpHeaders();
        }
        headers = headers.append('Content-Type', 'application/text');
        headers = headers.append('Content-encrypted', 'aes');
        return headers;
    }
    warn() {
        console.warn('Warning: You transfer non-encrypted content to the interface. Please check your settings');
    }
    configure(options) {
        this.secret = options.secret;
        this.warnInactive = options.warnInactive;
        this.cryptInactive = options.cryptInactive;
    }
    onLoad() {
        return this.isOnLoad.asObservable();
    }
    get(url, options) {
        this.isOnLoad.next(true);
        if (!this.cryptInactive) {
            options.headers = this.setCryptHeaders(options.headers);
            options.responseType = 'text';
        }
        return this.http.get(url, options).map(res => this.decrypt(res), e => this.isOnLoad.next(false));
    }
    patch(url, body, options) {
        this.isOnLoad.next(true);
        if (!this.cryptInactive) {
            options.headers = this.setCryptHeaders(options.headers);
            body = this.encrypt(body);
            options.responseType = 'text';
        }
        return this.http.patch(url, body, options).map(res => this.decrypt(res), e => this.isOnLoad.next(false));
    }
    post(url, body, options) {
        this.isOnLoad.next(true);
        if (!this.cryptInactive) {
            options.headers = this.setCryptHeaders(options.headers);
            options.responseType = 'text';
            body = this.encrypt(body);
        }
        return this.http.post(url, body, options).map(res => this.decrypt(res), e => this.isOnLoad.next(false));
    }
    put(url, body, options) {
        this.isOnLoad.next(true);
        if (!this.cryptInactive) {
            options.headers = this.setCryptHeaders(options.headers);
            body = this.encrypt(body);
            options.responseType = 'text';
        }
        return this.http.put(url, body, options).map(res => this.decrypt(res), e => this.isOnLoad.next(false));
    }
    delete(url, options) {
        this.isOnLoad.next(true);
        if (!this.cryptInactive) {
            options.headers = this.setCryptHeaders(options.headers);
            options.responseType = 'text';
        }
        return this.http.delete(url, options).map(res => this.decrypt(res), e => this.isOnLoad.next(false));
    }
    options(url, options) {
        this.isOnLoad.next(true);
        if (!this.cryptInactive) {
            options.headers = this.setCryptHeaders(options.headers);
            options.responseType = 'text';
        }
        return this.http.options(url, options).map(res => this.decrypt(res), e => this.isOnLoad.next(false));
    }
};
CryptHttpService = __decorate([
    Injectable(),
    __param(0, Inject(HttpClient)),
    __metadata("design:paramtypes", [HttpClient, CryptService])
], CryptHttpService);

let NgxHttpCryptModule = class NgxHttpCryptModule {
};
NgxHttpCryptModule = __decorate([
    NgModule({
        imports: [
            HttpClientModule
        ],
        providers: [
            CryptHttpService, CryptService
        ]
    })
], NgxHttpCryptModule);

export { CryptHttpService, CryptService, NgxHttpCryptModule };
//# sourceMappingURL=ngx-http-crypt.js.map
